@@ -34,13 +34,25 @@
 #include "check_impl.h"
 #include "check_pack.h"
 
-#ifdef HAVE_PTHREAD
-pthread_mutex_t lock_mutex = PTHREAD_MUTEX_INITIALIZER;
-#else
+#ifndef HAVE_PTHREAD
 #define pthread_mutex_lock(arg)
 #define pthread_mutex_unlock(arg)
+#define pthread_cleanup_push(f,a) {
+#define pthread_cleanup_pop(e) }
 #endif
 
+/* Maximum size for one message in the message stream. */
+#define CK_MAX_MSG_SIZE 8192
+/* This is used to implement a sliding window on the receiving
+ * side. When sending messages, we assure that no single message
+ * is bigger than this (actually we check against CK_MAX_MSG_SIZE/2).
+ * The usual size for a message is less than 80 bytes.
+ * All this is done instead of the previous approach to allocate (actually
+ * continuously reallocate) one big chunk for the whole message stream.
+ * Problems were seen in the wild with up to 4 GB reallocations.
+ */
+
+
 /* typedef an unsigned int that has at least 4 bytes */
 typedef uint32_t ck_uint32;
 
@@ -53,15 +65,17 @@ static char *upack_str  (char **buf);
 static int   pack_ctx   (char **buf, CtxMsg  *cmsg);
 static int   pack_loc   (char **buf, LocMsg  *lmsg);
 static int   pack_fail  (char **buf, FailMsg *fmsg);
+static int   pack_duration  (char **buf, DurationMsg *fmsg);
 static void  upack_ctx  (char **buf, CtxMsg  *cmsg);
 static void  upack_loc  (char **buf, LocMsg  *lmsg);
 static void  upack_fail (char **buf, FailMsg *fmsg);
+static void  upack_duration (char **buf, DurationMsg *fmsg);
 
 static void  check_type (int type, const char *file, int line);
 static enum ck_msg_type upack_type (char **buf);
 static void  pack_type  (char **buf, enum ck_msg_type type);
 
-static int   read_buf   (int fdes, char **buf);
+static int   read_buf   (int fdes, int size, char *buf);
 static int   get_result (char *buf, RcvMsg *rmsg);
 static void  rcvmsg_update_ctx (RcvMsg *rmsg, enum ck_result_ctx ctx);
 static void  rcvmsg_update_loc (RcvMsg *rmsg, const char *file, int line);
@@ -74,13 +88,15 @@ typedef void (*upfun) (char **, CheckMsg *);
 static pfun pftab [] = {
   (pfun) pack_ctx,
   (pfun) pack_fail,
-  (pfun) pack_loc
+  (pfun) pack_loc,
+  (pfun) pack_duration
 };
 
 static upfun upftab [] = {
   (upfun) upack_ctx,
   (upfun) upack_fail,
-  (upfun) upack_loc
+  (upfun) upack_loc,
+  (upfun) upack_duration
 };
 
 int pack (enum ck_msg_type type, char **buf, CheckMsg *msg)
@@ -98,7 +114,6 @@ int pack (enum ck_msg_type type, char **buf, CheckMsg *msg)
 int upack (char *buf, CheckMsg *msg, enum ck_msg_type *type)
 {
   char *obuf;
-  int nread;
 
   if (buf == NULL)
     return -1;
@@ -111,8 +126,7 @@ int upack (char *buf, CheckMsg *msg, enum ck_msg_type *type)
   
   upftab[*type] (&buf, msg);
 
-  nread = buf - obuf;
-  return nread;
+  return buf - obuf;
 }
 
 static void pack_int (char **buf, int val)
@@ -207,6 +221,25 @@ static void upack_ctx (char **buf, CtxMsg *cmsg)
   cmsg->ctx = upack_int (buf);
 }
 
+static int pack_duration (char **buf, DurationMsg *cmsg)
+{
+  char *ptr;
+  int len;
+
+  len = 4 + 4;
+  *buf = ptr = emalloc (len);
+
+  pack_type (&ptr, CK_MSG_DURATION);
+  pack_int (&ptr, cmsg->duration);
+
+  return len;
+}
+
+static void upack_duration (char **buf, DurationMsg *cmsg)
+{
+  cmsg->duration = upack_int (buf);
+}
+
 static int pack_loc (char **buf, LocMsg *lmsg)
 {
   char *ptr;
@@ -254,7 +287,11 @@ static void check_type (int type, const char *file, int line)
 }
 
 #ifdef HAVE_PTHREAD
-pthread_mutex_t mutex_lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t ck_mutex_lock = PTHREAD_MUTEX_INITIALIZER;
+void ppack_cleanup( void *mutex )
+{
+  pthread_mutex_unlock(mutex);
+}
 #endif
 
 void ppack (int fdes, enum ck_msg_type type, CheckMsg *msg)
@@ -264,42 +301,32 @@ void ppack (int fdes, enum ck_msg_type type, CheckMsg *msg)
   ssize_t r;
 
   n = pack (type, &buf, msg);
-  pthread_mutex_lock(&mutex_lock);
+  /* Keep it on the safe side to not send too much data. */
+  if (n > (CK_MAX_MSG_SIZE / 2))
+    eprintf("Message string too long", __FILE__, __LINE__ - 2);
+
+  pthread_cleanup_push( ppack_cleanup, &ck_mutex_lock );
+  pthread_mutex_lock(&ck_mutex_lock);
   r = write (fdes, buf, n);
-  pthread_mutex_unlock(&mutex_lock);
+  pthread_mutex_unlock(&ck_mutex_lock);
+  pthread_cleanup_pop(0);
   if (r == -1)
     eprintf ("Error in call to write:", __FILE__, __LINE__ - 2);
 
   free (buf);
 }
 
-static int read_buf (int fdes, char **buf)
+static int read_buf (int fdes, int size, char *buf)
 {
-  char *readloc;
   int n;
-  int nread = 0;
-  int size = 1;
-  int grow = 2;
-  
-  *buf = emalloc(size);
-  readloc = *buf;
-  while (1) {
-    n = read (fdes, readloc, size - nread);
-    if (n == 0)
-      break;
-    if (n == -1)
-      eprintf ("Error in call to read:", __FILE__, __LINE__ - 4);
 
-    nread += n;
-    size *= grow;
-    *buf = erealloc (*buf,size);
-    readloc = *buf + nread;
-  }
+  n = read (fdes, buf, size);
+  if (n == -1)
+    eprintf ("Error in call to read:", __FILE__, __LINE__ - 4);
 
-  return nread;
+  return n;
 }    
 
-
 static int get_result (char *buf, RcvMsg *rmsg)
 {
   enum ck_msg_type type;
@@ -333,6 +360,9 @@ static int get_result (char *buf, RcvMsg *rmsg)
       /* Skip subsequent failure messages, only happens for CK_NOFORK */
     }
     free (fmsg->msg);
+  } else if (type == CK_MSG_DURATION) {
+    DurationMsg *cmsg = (DurationMsg *) &msg;
+    rmsg->duration = cmsg->duration;
   } else
     check_type (type, __FILE__, __LINE__);
 
@@ -359,6 +389,7 @@ static RcvMsg *rcvmsg_create (void)
   rmsg->lastctx = CK_CTX_INVALID;
   rmsg->failctx = CK_CTX_INVALID;
   rmsg->msg = NULL;
+  rmsg->duration = -1;
   reset_rcv_test (rmsg);
   reset_rcv_fixture (rmsg);
   return rmsg;
@@ -401,22 +432,33 @@ static void rcvmsg_update_loc (RcvMsg *rmsg, const char *file, int line)
   
 RcvMsg *punpack (int fdes)
 {
-  int nread, n;
+  int nread, nparse, n;
   char *buf;
-  char *obuf;
   RcvMsg *rmsg;
 
-  nread = read_buf (fdes, &buf);
-  obuf = buf;
   rmsg = rcvmsg_create ();
   
-  while (nread > 0) {
+  /* Allcate a buffer */
+  buf = emalloc(CK_MAX_MSG_SIZE);
+  /* Fill the buffer from the file */
+  nread = read_buf (fdes, CK_MAX_MSG_SIZE, buf);
+  nparse = nread;
+  /* While not all parsed */
+  while (nparse > 0) {
+    /* Parse one message */
     n = get_result (buf, rmsg);
-    nread -= n;
-    buf += n;
+    nparse -= n;
+    /* Move remaining data in buffer to the beginning */
+    memmove(buf, buf + n, nparse);
+    /* If EOF has not been seen */
+    if (nread > 0) {
+      /* Read more data into empty space at end of the buffer */
+      nread = read_buf (fdes, n, buf + nparse);
+      nparse += nread;
+    }
   }
+  free (buf);
 
-  free (obuf);
   if (rmsg->lastctx == CK_CTX_INVALID) {
     free (rmsg);
     rmsg = NULL;
