@@ -88,6 +88,37 @@ static void srunner_fprint_results (FILE *file, SRunner *sr,
   return;
 }
 
+void fprint_xml_esc(FILE *file, const char *str)
+{
+  for (; *str != '\0'; str++) {
+
+    switch (*str) {
+
+    /* handle special characters that must be escaped */
+    case '"':
+      fputs("&quot;", file);
+      break;
+    case '\'':
+      fputs("&apos;", file);
+      break;
+    case '<':
+      fputs("&lt;", file);
+      break;
+    case '>':
+      fputs("&gt;", file);
+      break;
+    case '&':
+      fputs("&amp;", file);
+      break;
+
+    /* regular characters, print as is */
+    default:
+      fputc(*str, file);
+      break;
+    }
+  }
+}
+
 void tr_fprint (FILE *file, TestResult *tr, enum print_output print_mode)
 {
   if (print_mode == CK_ENV) {
@@ -105,9 +136,9 @@ void tr_fprint (FILE *file, TestResult *tr, enum print_output print_mode)
 void tr_xmlprint (FILE *file, TestResult *tr, enum print_output print_mode CK_ATTRIBUTE_UNUSED)
 {
   char result[10];
-  char *path_name;
-  char *file_name;
-  char *slash;
+  char *path_name = "";
+  char *file_name = "";
+  char *slash = NULL;
 
   switch (tr->rtype) {
   case CK_PASS:
@@ -125,15 +156,17 @@ void tr_xmlprint (FILE *file, TestResult *tr, enum print_output print_mode CK_AT
     break;
   }
 
-  slash = strrchr(tr->file, '/');
-  if (slash == NULL) {
-    path_name = (char*)".";
-    file_name = tr->file;
-  } else {
-    path_name = strdup(tr->file);
-    path_name[slash - tr->file] = 0; /* Terminate the temporary string. */
-    file_name = slash + 1;
-  }
+  if (tr->file) {
+    slash = strrchr(tr->file, '/');
+    if (slash == NULL) {
+      path_name = (char*)".";
+      file_name = tr->file;
+    } else {
+      path_name = strdup(tr->file);
+      path_name[slash - tr->file] = 0; /* Terminate the temporary string. */
+      file_name = slash + 1;
+    }
+  } 
     
 
   fprintf(file, "    <test result=\"%s\">\n", result);
@@ -141,8 +174,15 @@ void tr_xmlprint (FILE *file, TestResult *tr, enum print_output print_mode CK_AT
   fprintf(file, "      <fn>%s:%d</fn>\n", file_name, tr->line);
   fprintf(file, "      <id>%s</id>\n", tr->tname);
   fprintf(file, "      <iteration>%d</iteration>\n", tr->iter);
-  fprintf(file, "      <description>%s</description>\n", tr->tcname);
-  fprintf(file, "      <message>%s</message>\n", tr->msg);
+  fprintf(file, "      <duration>%d.%06d</duration>\n",
+          tr->duration < 0 ? -1 : tr->duration / 1000000,
+          tr->duration < 0 ? 0 : tr->duration % 1000000);
+  fprintf(file, "      <description>");
+  fprint_xml_esc(file, tr->tcname);
+  fprintf(file,"</description>\n", tr->tcname);
+  fprintf(file, "      <message>", tr->msg);
+  fprint_xml_esc(file, tr->msg);
+  fprintf(file,"</message>\n", tr->msg);
   fprintf(file, "    </test>\n");
   
   if (slash != NULL) {
