@@ -1,4 +1,8 @@
-#include <time.h>
+#include "../lib/libcompat.h"
+
+/* note: this test appears pretty useless, so we aren't including it
+   in the TESTS variable of Makefile.am */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <check.h>
@@ -42,7 +46,6 @@ static void run (int num_iters)
   }
 
   srunner_free(sr);
-  suite_free(s);
 }
 
   
